package main

import (
	"bufio"
	"fmt"
	"math/rand"
	"os"
	"time"
)

const (
	LENGTH = 5      // length of the magic sequence
	MAXLETTER = 'F' // maximum letter that a magic sequence can contain
	SPECIAL = '*'   // a special character that is used to replace a used character
)


/* Returns a random uppercase latin letter <= MAXLETTER. */
func randomLetter() rune {
	return 'A' + rune(rand.Intn(MAXLETTER - 'A' + 1))
}

/* Returns a random magic sequence of uppercase latin letters. */
func randomSequence() string {
	s := ""
	for i := 0; i < LENGTH; i++ {
		s += string(randomLetter())
	}
	return s
}

/* Compares the magic sequence m with the guess g, and returns the number of letters that are correct and in the correct position, and those that are correct but in the wrong position. */
func compare(m, g string) (okcp int, okwp int) {
	resm := []byte{}
	resg := []byte{}
	for i := 0; i < LENGTH; i++ {
		if m[i] == g[i] {
			okcp++
		} else {
			resm = append(resm, m[i])
			resg = append(resg, g[i])
		}
	}
	for _, cg := range resg {
		var i int 
		for i = 0; i < len(resm); i++ {
			if cg == resm[i] {
				resm[i] = SPECIAL
				okwp++
				break
			}
		}
	}
	return
}

func main() {
	rand.Seed(time.Now().UTC().UnixNano())
	magic := randomSequence()
	b := bufio.NewScanner(os.Stdin)
	attempts := 0
	fmt.Printf("Your guess: ")
	for b.Scan() {
		guess := b.Text()
		if len(guess) != LENGTH {
			fmt.Printf("Wrong length (should be %d)\n", LENGTH)
		} else {
			okcp, okwp := compare(magic, guess)
			attempts++
			fmt.Printf("\tResult: %3d, %3d\n", okcp, okwp)
			if okcp == LENGTH {
				break
			}
		}
		fmt.Printf("Your guess: ")
	}
	fmt.Printf("You succeeded in %d attempts!\n", attempts)
}
