package main

import (
	"fmt"
	"math/rand"
	"time"
)

const (
	LENGTH = 5      // length of the magic sequence
	MAXLETTER = 'F' // maximum letter that a magic sequence can contain
	SPECIAL = '*'   // a special character that is used to replace a used character
)


/* Returns a random uppercase latin letter <= MAXLETTER. */
func randomLetter() rune {
	return 'A' + rune(rand.Intn(MAXLETTER - 'A' + 1))
}

/* Returns a random magic sequence of uppercase latin letters. */
func randomSequence() string {
	s := ""
	for i := 0; i < LENGTH; i++ {
		s += string(randomLetter())
	}
	return s
}


type annaInteraction struct {
	guess string
	okcp, okwp int
}

/* Compares the magic sequence m with the guess g, and returns the number of letters that are correct and in the correct position, and those that are correct but in the wrong position. */
func compare(m, g string) (okcp int, okwp int) {
	resm := []byte{}
	resg := []byte{}
	for i := 0; i < LENGTH; i++ {
		if m[i] == g[i] {
			okcp++
		} else {
			resm = append(resm, m[i])
			resg = append(resg, g[i])
		}
	}
	for _, cg := range resg {
		var i int 
		for i = 0; i < len(resm); i++ {
			if cg == resm[i] {
				resm[i] = SPECIAL
				okwp++
				break
			}
		}
	}
	return
}



func guessInternal(interactions []annaInteraction, prefix string) (res string, found bool) {
	if len(prefix) == LENGTH {
		for _, interaction := range interactions {
			okcp, okwp := compare(interaction.guess, prefix)
			if okcp != interaction.okcp || okwp != interaction.okwp {
				return "", false
			}
		}
		return prefix, true
	}
	for r := 'A'; r <= MAXLETTER; r++ {
		res, found = guessInternal(interactions, prefix + string(r))
		if found {
			return
		}
	}
	return 
}

func guess(interactions []annaInteraction) string {
	res, _ := guessInternal(interactions, "")
	return res
}


func main() {
	rand.Seed(time.Now().UTC().UnixNano())
	magic := randomSequence()
	fmt.Printf("Anna's magic sequence: %s\n", magic)
	attempts := 0
	var interactions []annaInteraction
	for {
		attempts++
		beppeGuess := guess(interactions)
		fmt.Printf("Beppe's guess: %s\n", beppeGuess)
		okcp, okwp := compare(magic, beppeGuess)
		fmt.Printf("\tResult: %d\t%d\n", okcp, okwp)
		interactions = append(interactions, annaInteraction{beppeGuess, okcp, okwp})
		if okcp == LENGTH {
			break
		}
	}
	
	fmt.Printf("Beppe succeeded in %d attempts!\n", attempts)
}
