package main

import (
  "fmt"
  "image"
  "image/png"
  "github.com/holizz/terrapin"
  "math"
  "net/http"
)

func pageMain(w http.ResponseWriter, r *http.Request) {
  fmt.Fprint(w, `
      <!doctype html>
      <title>Fiocco di Koch</title>
      <h1>Fiocco di Koch</h1>
      <img src="/koch.png">
      `)
}

func kochCurve(t *terrapin.Terrapin, lung float64, liv int) {
  if liv == 0 {
    t.Forward(lung)
    return
  }
  kochCurve(t, lung, liv - 1)
  t.Left(math.Pi / 3.0)
  kochCurve(t, lung, liv - 1)
  t.Right(math.Pi - math.Pi / 3.0)
  kochCurve(t, lung, liv - 1)
  t.Left(math.Pi / 3.0)
  kochCurve(t, lung, liv - 1)
}

func kochSnowFlake(t *terrapin.Terrapin, lung float64, liv int) {
  for i := 0; i < 3; i++ {
    kochCurve(t, lung, liv)
    t.Right(2.0 * math.Pi / 3.0)
  }  
}

func pageImmagine(w http.ResponseWriter, r *http.Request) {
  i := image.NewRGBA(image.Rect(0, 0, 500, 500))
  t := terrapin.NewTerrapin(i, terrapin.Position{200.0, 450.0})
  kochSnowFlake(t, 4.0, 4)
  png.Encode(w, i)
}


func main() {
  http.HandleFunc("/koch.html", pageMain)
  http.HandleFunc("/koch.png", pageImmagine)
  fmt.Println("Listening on http://localhost:3000/")
  http.ListenAndServe(":3000", nil)
}
